//! RustDDS-Gen: IDL 到 Rust 代码生成器
//!
//! 该工具用于将 IDL（接口定义语言）文件转换为 Rust 代码。
//! 主要用于 RustDDS 项目，支持解析 IDL 文件并生成相应的 Rust 结构体、枚举、常量等。
//! 提供了命令行界面，支持多种选项，如调试输出、仅解析不生成代码等。

use clap::Parser;
use std::path::PathBuf;
use anyhow::{Result, Context};
use std::fs;
use clap::Parser as ClapParser;

#[macro_use]
mod parser;
mod codegen;

/// 命令行参数解析结构体
/// 
/// 定义了工具支持的所有命令行选项。
#[derive(ClapParser)]
#[command(name = "RustDDS-Gen")]
#[command(about = "IDL到Rust代码生成器，用于RustDDS项目")]
struct Cli {
    /// 输入IDL文件
    #[arg(short, long)]
    input: PathBuf,
    
    /// 输出Rust文件
    #[arg(short, long)]
    output: PathBuf,
    
    /// 是否启用调试输出
    #[arg(short, long)]
    debug: bool,

    /// 只解析语法，不生成代码
    #[arg(long)]
    parse_only: bool,

    /// 打印详细的解析过程
    #[arg(long)]
    verbose: bool,

    /// 打印语法树
    #[arg(long)]
    print_ast: bool,

    /// 将解析结果保存到文件
    #[arg(long)]
    dump_ast: Option<PathBuf>,
}

/// 主函数
/// 
/// 解析命令行参数，根据参数执行相应的操作：
/// 1. 解析 IDL 文件
/// 2. 可选地打印或保存解析结果
/// 3. 生成 Rust 代码（除非指定了 --parse-only）
/// 
/// # 返回值
/// * `Result<()>` - 成功时返回 `()`，失败时返回错误
fn main() -> Result<()> {
    // 解析命令行参数
    let cli = Cli::parse();

    // 设置环境变量以控制调试输出
    if cli.verbose {
        std::env::set_var("RUST_DDS_IDL_VERBOSE", "1");
    }
    
    // 只在调试模式下输出基本信息
    println!("处理IDL文件: {:?}", cli.input);
    
    // 调用解析函数，启用详细日志
    let idl_file = if cli.verbose {
        // 使用详细模式解析
        parser::parse_idl_file_verbose(&cli.input)
            .with_context(|| format!("解析IDL文件失败: {:?}", cli.input))?
    } else {
        // 使用标准模式解析
        parser::parse_idl_file(&cli.input)
            .with_context(|| format!("解析IDL文件失败: {:?}", cli.input))?
    };
    
    // 如果需要打印AST，使用更简洁的方式
    if cli.print_ast {
        debug_println!("解析结果摘要: {} 个模块, {} 个结构体, {} 个枚举", 
            idl_file.modules.len(), idl_file.structs.len(), idl_file.enums.len());
    }
    
    // 如果指定了dump_ast选项，将AST保存到文件
    if let Some(ast_file) = &cli.dump_ast {
        let ast_json = serde_json::to_string_pretty(&idl_file)
            .with_context(|| "无法序列化AST为JSON")?;
        fs::write(ast_file, ast_json)
            .with_context(|| format!("无法写入AST到文件: {:?}", ast_file))?;
    }
    
    // 如果只是解析，不生成代码，则直接返回
    if cli.parse_only {
        debug_println!("解析完成，不生成代码");
        return Ok(());
    }
    
    // 生成Rust代码
    println!("生成Rust代码到: {:?}", cli.output);
    codegen::generate_rust_code(&idl_file, &cli.output)
        .with_context(|| format!("生成Rust代码失败: {:?}", cli.output))?;
    
    // 简化的处理完成信息
    println!("处理完成: {} 个模块, {} 个结构体, {} 个枚举, {} 个常量, {} 个类型定义", 
        idl_file.modules.len(), idl_file.structs.len(), idl_file.enums.len(), 
        idl_file.constants.len(), idl_file.typedefs.len());
    
    Ok(())
}
