use pest::iterators::Pair;
use crate::parser::{IdlInterface, IdlMethod, IdlParameter, ParameterDirection};
use super::Rule;
/// 处理接口定义
pub fn process_interface(pair: Pair<Rule>) -> IdlInterface {
    let mut interface = IdlInterface {
        name: String::new(),
        parent: None,
        methods: Vec::new(),
        annotations: Vec::new(),
    };
    
    // 遍历接口定义内部元素
    let mut identifiers = Vec::new();
    
    for inner_pair in pair.into_inner() {
        match inner_pair.as_rule() {
            Rule::identifier => {
                identifiers.push(inner_pair.as_str().to_string());
            },
            Rule::interface_member => {
                // 处理接口成员（方法）
                for member_pair in inner_pair.into_inner() {
                    if member_pair.as_rule() == Rule::method_def {
                        let method = process_method(member_pair);
                        interface.methods.push(method);
                    }
                }
            },
            Rule::annotation => {
                // 处理注解，暂时不实现
            },
            _ => {
                // 忽略未知元素
                ()
            }
        }
    }
    
    // 处理接口名称和父接口
    if !identifiers.is_empty() {
        interface.name = identifiers[0].clone();
        // 接口名称已设置
        
        // 如果有第二个标识符，则这是父接口
        if identifiers.len() > 1 {
            interface.parent = Some(identifiers[1].clone());
            // 父接口已设置
        }
    }
    
    interface
}

/// 处理方法定义
fn process_method(pair: Pair<Rule>) -> IdlMethod {
    let mut method = IdlMethod {
        name: String::new(),
        return_type: String::new(),
        parameters: Vec::new(),
        annotations: Vec::new(),
    };
    
    // 遍历方法定义内部元素
    let inner_pairs = pair.into_inner().collect::<Vec<_>>();
    
    // 第一个元素应该是返回类型
    if !inner_pairs.is_empty() {
        let return_type_pair = &inner_pairs[0];
        if return_type_pair.as_rule() == Rule::type_spec {
            method.return_type = return_type_pair.as_str().to_string();
            // 方法返回类型已设置
        }
    }
    
    // 第二个元素应该是方法名
    if inner_pairs.len() > 1 {
        let name_pair = &inner_pairs[1];
        if name_pair.as_rule() == Rule::identifier {
            method.name = name_pair.as_str().to_string();
            // 方法名称已设置
        }
    }
    
    // 处理参数列表
    if inner_pairs.len() > 2 {
        for i in 2..inner_pairs.len() {
            let param_pair = &inner_pairs[i];
            if param_pair.as_rule() == Rule::method_param_list {
                for param in param_pair.clone().into_inner() {
                    let parameter = process_parameter(param);
                    method.parameters.push(parameter);
                }
            }
        }
    }
    
    method
}

/// 处理参数定义
fn process_parameter(pair: Pair<Rule>) -> IdlParameter {
    let mut parameter = IdlParameter {
        name: String::new(),
        type_spec: String::new(),
        direction: ParameterDirection::In, // 默认为in
    };
    
    // 简化的参数定义信息
    let param_str = pair.as_str().to_string();
    
    // 从参数字符串中检测方向
    if param_str.starts_with("in ") {
        parameter.direction = ParameterDirection::In;
    } else if param_str.starts_with("out ") {
        parameter.direction = ParameterDirection::Out;
    } else if param_str.starts_with("inout ") {
        parameter.direction = ParameterDirection::InOut;
    } else {
        parameter.direction = ParameterDirection::In; // 默认为in
        // 默认参数方向已设置
    }
    
    // 遍历参数定义内部元素
    let inner_pairs = pair.into_inner().collect::<Vec<_>>();
    
    // 先找到类型和名称，然后再处理方向
    let mut type_index = None;
    let mut name_index = None;
    
    // 首先寻找类型和名称的索引
    for (i, inner_pair) in inner_pairs.iter().enumerate() {
        match inner_pair.as_rule() {
            Rule::type_spec => {
                type_index = Some(i);
            },
            Rule::identifier => {
                name_index = Some(i);
            },
            _ => {}
        }
    }
    
    // 提取类型和名称
    if let Some(i) = type_index {
        parameter.type_spec = inner_pairs[i].as_str().to_string();
        // 参数类型已设置
    }
    
    if let Some(i) = name_index {
        parameter.name = inner_pairs[i].as_str().to_string();
        // 参数名称已设置
    }
    
    // 如果没有找到类型或名称，尝试使用简单的字符串匹配
    if parameter.type_spec.is_empty() || parameter.name.is_empty() {
        // 分割参数字符串，忽略方向部分
        let param_parts = param_str.trim().split_whitespace().collect::<Vec<_>>();
        if param_parts.len() >= 2 {
            // 如果开头是方向，则跳过
            let start_index = if param_parts[0] == "in" || param_parts[0] == "out" || param_parts[0] == "inout" {
                1
            } else {
                0
            };
            
            if start_index < param_parts.len() && parameter.type_spec.is_empty() {
                parameter.type_spec = param_parts[start_index].to_string();
                // 从字符串提取的参数类型已设置
            }
            
            if start_index + 1 < param_parts.len() && parameter.name.is_empty() {
                parameter.name = param_parts[start_index + 1].to_string();
                // 从字符串提取的参数名称已设置
            }
        }
    }
    
    parameter
}
