//! IDL 解析器模块
//!
//! 该模块负责解析 IDL 文件，并将其转换为内部数据结构以便于后续的代码生成。
//! 使用 pest 库进行语法解析，并定义了各种 IDL 结构的数据类型。

use serde::{Serialize, Deserialize};
use std::fs;
use std::path::Path;
use anyhow::{Result, Context, anyhow};
use thiserror::Error;
use pest::iterators::Pair;

/// 调试输出宏
/// 
/// 只在设置了 RUST_DDS_IDL_VERBOSE 环境变量时才会输出信息
/// 这样可以在需要时启用详细的调试输出，而在正常运行时保持安静
#[macro_export]
macro_rules! debug_println {
    ($($arg:tt)*) => {
        if std::env::var("RUST_DDS_IDL_VERBOSE").is_ok() {
            println!($($arg)*);
        }
    };
}

// 引入子模块
mod union;      // 处理 union 类型的解析
mod interface;  // 处理接口类型的解析
use pest_derive::Parser;

#[derive(Parser)]
#[grammar = "parser/idl.pest"]
pub struct IdlParser;

// 导出Rule类型，供其他模块使用
// Rule类型已经通过pest_derive::Parser自动引入

impl IdlParser {
    pub fn parse(input: &str) -> Result<pest::iterators::Pairs<'_, Rule>, pest::error::Error<Rule>> {
        <Self as pest::Parser<Rule>>::parse(Rule::idl_file, input)
    }
}

#[derive(Debug, Error)]
pub enum IdlError {
    #[error("IO错误: {0}")]
    IoError(#[from] std::io::Error),
}

// 注意：Rule枚举由pest自动生成

/// IDL文件的表示
#[derive(Debug, Clone, Serialize, Deserialize)]
pub struct IdlFile {
    pub modules: Vec<IdlModule>,
    pub structs: Vec<IdlStruct>,
    pub enums: Vec<IdlEnum>,
    pub typedefs: Vec<IdlTypedef>,
    pub constants: Vec<IdlConstant>,
}

/// IDL模块的表示
#[derive(Debug, Clone, Serialize, Deserialize)]
// 模块内元素的类型
pub enum ModuleItem {
    Struct(IdlStruct),
    Enum(IdlEnum),
    Typedef(IdlTypedef),
    Constant(IdlConstant),
    Union(IdlUnion),
    Interface(IdlInterface),
    Module(IdlModule),
}

#[derive(Debug, Clone, Serialize, Deserialize)]
pub struct IdlModule {
    pub name: String,
    pub structs: Vec<IdlStruct>,
    pub enums: Vec<IdlEnum>,
    pub typedefs: Vec<IdlTypedef>,
    pub constants: Vec<IdlConstant>,
    pub unions: Vec<IdlUnion>,
    pub interfaces: Vec<IdlInterface>,
    pub modules: Vec<IdlModule>,  // 添加嵌套模块支持
    pub items: Vec<ModuleItem>,   // 按原始顺序存储所有元素
}

/// IDL结构体的表示
#[derive(Debug, Clone, Serialize, Deserialize)]
pub struct IdlStruct {
    pub name: String,
    pub fields: Vec<IdlField>,
    pub annotations: Vec<IdlAnnotation>,
}

/// IDL字段的表示
#[derive(Debug, Clone, Serialize, Deserialize)]
pub struct IdlField {
    pub name: String,
    pub type_spec: String,
    pub array_size: Option<usize>,
    pub annotations: Vec<IdlAnnotation>,
}

/// IDL枚举的表示
#[derive(Debug, Clone, Serialize, Deserialize)]
pub struct IdlEnum {
    pub name: String,
    pub values: Vec<IdlEnumValue>,
    pub annotations: Vec<IdlAnnotation>,
}

/// IDL枚举值的表示
#[derive(Debug, Clone, Serialize, Deserialize)]
pub struct IdlEnumValue {
    pub name: String,
    pub value: Option<i32>,
}

/// IDL类型定义的表示
#[derive(Debug, Clone, Serialize, Deserialize)]
pub struct IdlTypedef {
    pub name: String,
    pub type_spec: String,
}

/// IDL常量的表示
#[derive(Debug, Clone, Serialize, Deserialize)]
pub struct IdlConstant {
    pub name: String,
    pub type_spec: String,
    pub value: IdlConstValue,
}

#[derive(Debug, Clone, serde::Serialize, serde::Deserialize)]
pub enum IdlConstValue {
    Integer(i64),
    Float(f64),
    String(String),
    Boolean(bool),
}

/// IDL联合类型的表示
#[derive(Debug, Clone, Serialize, Deserialize)]
pub struct IdlUnion {
    pub name: String,
    pub discriminator_type: String,
    pub cases: Vec<IdlUnionCase>,
}

/// IDL联合类型分支的表示
#[derive(Debug, Clone, Serialize, Deserialize)]
pub struct IdlUnionCase {
    pub labels: Vec<String>,  // case标签，可能有多个或者是default
    pub is_default: bool,     // 是否是default分支
    pub field: IdlField,      // 该分支对应的字段
}

/// IDL接口的表示
#[derive(Debug, Clone, Serialize, Deserialize)]
pub struct IdlInterface {
    pub name: String,
    pub parent: Option<String>,
    pub methods: Vec<IdlMethod>,
    pub annotations: Vec<IdlAnnotation>,
}

/// IDL接口方法的表示
#[derive(Debug, Clone, Serialize, Deserialize)]
pub struct IdlMethod {
    pub name: String,
    pub return_type: String,
    pub parameters: Vec<IdlParameter>,
    pub annotations: Vec<IdlAnnotation>,
}

/// IDL方法参数的表示
#[derive(Debug, Clone, Serialize, Deserialize)]
pub struct IdlParameter {
    pub name: String,
    pub type_spec: String,
    pub direction: ParameterDirection,
}

/// IDL 参数方向
/// 
/// 表示 IDL 接口方法参数的方向，可以是输入、输出或输入输出。
#[derive(Debug, Clone, Serialize, Deserialize)]
pub enum ParameterDirection {
    In,
    Out,
    InOut,
}

/// IDL注解的表示
#[derive(Debug, Clone, Serialize, Deserialize)]
pub struct IdlAnnotation {
    pub name: String,
    pub params: Option<IdlAnnotationParam>,
}

/// IDL注解参数的表示
#[derive(Debug, Clone, Serialize, Deserialize)]
pub enum IdlAnnotationParam {
    Value(IdlAnnotationValue),
    KeyValue(Vec<(String, IdlAnnotationValue)>),
}

/// IDL注解值的表示
#[derive(Debug, Clone, Serialize, Deserialize)]
pub enum IdlAnnotationValue {
    String(String),
    Integer(i64),
    Float(f64),
    Boolean(bool),
    Identifier(String),
}

impl IdlFile {
}

/// 解析 IDL 文件
/// 
/// 该函数是解析器的主要入口点，负责读取指定路径的 IDL 文件，
/// 并将其解析为内部数据结构 IdlFile。
/// 
/// # 参数
/// * `file_path` - IDL 文件的路径
/// 
/// # 返回值
/// * `Result<IdlFile>` - 成功时返回解析后的 IDL 文件结构，失败时返回错误
/// 
/// # 示例
/// ```no_run
/// use rust_dds_idl::parser::parse_idl_file;
/// 
/// let idl_file = parse_idl_file("example.idl").expect("Failed to parse IDL file");
/// ```
pub fn parse_idl_file<P: AsRef<Path>>(file_path: P) -> Result<IdlFile> {
    let file_content = fs::read_to_string(&file_path)
        .with_context(|| format!("无法读取IDL文件: {:?}", file_path.as_ref()))?;
    
    // 简化的基本信息
    debug_println!("解析IDL文件: {:?}", file_path.as_ref());
        
    // 使用pest解析IDL文件
    let pairs = IdlParser::parse(&file_content)
        .map_err(|e| anyhow!("解析IDL文件失败: {}", e))?;
    

    
    // 将解析结果转换为自定义数据结构
    let mut idl_file = IdlFile {
        modules: Vec::new(),
        structs: Vec::new(),
        enums: Vec::new(),
        typedefs: Vec::new(),
        constants: Vec::new(),
    };
    
    // 处理解析结果
    for pair in pairs {
        match pair.as_rule() {
            Rule::idl_file => {
                for inner_pair in pair.into_inner() {
                    process_top_level_item(&mut idl_file, inner_pair);
                }
            },
            _ => {
                // 忽略意外的解析规则
                ()
            }
        }
    }
    
    // 简化的解析完成信息
    debug_println!("解析完成: {} 模块, {} 结构体, {} 枚举, {} 常量, {} 类型定义", 
        idl_file.modules.len(), idl_file.structs.len(), idl_file.enums.len(), 
        idl_file.constants.len(), idl_file.typedefs.len());
    
    Ok(idl_file)
}



/// 解析模块定义
/// 
/// 将 pest 解析器生成的模块定义转换为内部的 IdlModule 结构。
/// 
/// # 参数
/// * `pair` - pest 解析器生成的模块定义节点
/// 
/// # 返回值
/// * `IdlModule` - 解析后的模块结构
fn parse_module(pair: pest::iterators::Pair<Rule>) -> IdlModule {
    // 简化的模块解析信息
    let mut inner_pairs = pair.into_inner();
    
    // 获取模块名
    let name = inner_pairs.next().unwrap().as_str().to_string();
    debug_println!("解析模块: {}", name);
    
    let mut module = IdlModule {
        name,
        structs: Vec::new(),
        enums: Vec::new(),
        typedefs: Vec::new(),
        constants: Vec::new(),
        unions: Vec::new(),
        interfaces: Vec::new(),
        modules: Vec::new(),
        items: Vec::new(), // 初始化 items 字段
    };
    
    // 遍历模块内容
    for item_pair in inner_pairs {
        debug_println!("处理模块内容项: {:?}", item_pair.as_rule());
        
        match item_pair.as_rule() {
            Rule::struct_def => {
                debug_println!("发现模块内的结构体定义");
                let struct_def = parse_struct(item_pair);
                debug_println!("解析到模块内的结构体: {}", struct_def.name);
                module.structs.push(struct_def);
            }
            Rule::enum_def => {
                debug_println!("发现模块内的枚举定义");
                let enum_def = parse_enum(item_pair);
                debug_println!("解析到模块内的枚举: {}", enum_def.name);
                module.enums.push(enum_def);
            }
            Rule::typedef_def => {
                debug_println!("发现模块内的类型定义");
                let typedef = parse_typedef(item_pair);
                debug_println!("解析到模块内的类型定义: {}", typedef.name);
                module.typedefs.push(typedef);
            }
            Rule::const_def => {
                debug_println!("发现模块内的常量定义");
                let constant = parse_constant(item_pair);
                debug_println!("解析到模块内的常量: {}", constant.name);
                module.constants.push(constant);
            }
            Rule::pragma_def => {
                debug_println!("忽略模块内的pragma定义");
            }
            Rule::interface_def => {
                debug_println!("忽略模块内的接口定义");
            }
            Rule::union_def => {
                debug_println!("忽略模块内的联合类型定义");
            }
            _ => debug_println!("忽略模块内的未知规则: {:?}", item_pair.as_rule()),
        }
    }
    
    debug_println!("模块解析完成，结构体数: {}, 枚举数: {}, 常量数: {}", 
             module.structs.len(), module.enums.len(), module.constants.len());
    
    module
}

// 解析结构体定义
fn parse_struct(pair: pest::iterators::Pair<Rule>) -> IdlStruct {
    let mut inner_pairs = pair.into_inner();
    
    // 解析注解
    let mut annotations = Vec::new();
    let mut name = String::new();
    
    for inner_pair in inner_pairs.by_ref() {
        match inner_pair.as_rule() {
            Rule::annotation => {
                let annotation = parse_annotation(inner_pair);
                annotations.push(annotation);
            }
            Rule::identifier => {
                name = inner_pair.as_str().to_string();
                break;
            }
            _ => (),
        }
    }
    
    // 解析字段
    let mut fields = Vec::new();
    
    for inner_pair in inner_pairs {
        if inner_pair.as_rule() == Rule::member_def {
            let field = parse_field(inner_pair);
            fields.push(field);
        }
    }
    
    IdlStruct {
        name,
        fields,
        annotations,
    }
}

// 解析字段定义
fn parse_field(pair: pest::iterators::Pair<Rule>) -> IdlField {
    let inner_pairs = pair.into_inner();
    
    // 解析注解
    let mut annotations = Vec::new();
    let mut type_spec = String::new();
    let mut name = String::new();
    let mut array_size = None;
    
    for inner_pair in inner_pairs {
        match inner_pair.as_rule() {
            Rule::annotation => {
                let annotation = parse_annotation(inner_pair);
                annotations.push(annotation);
            }
            Rule::type_spec => {
                type_spec = inner_pair.as_str().to_string();
            }
            Rule::identifier => {
                name = inner_pair.as_str().to_string();
            }
            Rule::array_spec => {
                let array_pair = inner_pair.into_inner().next().unwrap();
                array_size = Some(array_pair.as_str().parse().unwrap());
            }
            _ => (),
        }
    }
    
    IdlField {
        name,
        type_spec,
        array_size,
        annotations,
    }
}

// 解析枚举定义
fn parse_enum(pair: pest::iterators::Pair<Rule>) -> IdlEnum {
    let mut inner_pairs = pair.into_inner();
    
    // 解析注解
    let mut annotations = Vec::new();
    let mut name = String::new();
    
    for inner_pair in inner_pairs.by_ref() {
        match inner_pair.as_rule() {
            Rule::annotation => {
                let annotation = parse_annotation(inner_pair);
                annotations.push(annotation);
            }
            Rule::identifier => {
                name = inner_pair.as_str().to_string();
                break;
            }
            _ => (),
        }
    }
    
    // 解析枚举值
    let mut values = Vec::new();
    
    for inner_pair in inner_pairs {
        if inner_pair.as_rule() == Rule::enum_value_def {
            let value = parse_enum_value(inner_pair);
            values.push(value);
        }
    }
    
    IdlEnum {
        name,
        values,
        annotations,
    }
}

// 解析枚举值
fn parse_enum_value(pair: pest::iterators::Pair<Rule>) -> IdlEnumValue {
    let mut inner_pairs = pair.into_inner();
    
    let name = inner_pairs.next().unwrap().as_str().to_string();
    let value = inner_pairs.next().map(|p| p.as_str().parse().unwrap());
    
    IdlEnumValue {
        name,
        value,
    }
}

// 解析类型定义
fn parse_typedef(pair: pest::iterators::Pair<Rule>) -> IdlTypedef {
    let mut inner_pairs = pair.into_inner();
    
    let type_spec = inner_pairs.next().unwrap().as_str().to_string();
    let name = inner_pairs.next().unwrap().as_str().to_string();
    
    IdlTypedef {
        name,
        type_spec,
    }
}

// 解析常量定义
fn parse_constant(pair: pest::iterators::Pair<Rule>) -> IdlConstant {
    debug_println!("开始解析常量定义: {}", pair.as_str());
    debug_println!("常量定义规则: {:?}", pair.as_rule());
    
    // 打印所有内部元素
    let inner_pairs_debug: Vec<_> = pair.clone().into_inner().collect();
    for (i, p) in inner_pairs_debug.iter().enumerate() {
        debug_println!("内部元素 {}: 规则={:?}, 值={}", i, p.as_rule(), p.as_str());
    }
    
    let mut inner_pairs = pair.into_inner();
    
    let type_pair = inner_pairs.next().unwrap();
    let type_spec = type_pair.as_str().to_string();
    debug_println!("常量类型: {}, 规则: {:?}", type_spec, type_pair.as_rule());
    
    let name_pair = inner_pairs.next().unwrap();
    let name = name_pair.as_str().to_string();
    debug_println!("常量名称: {}, 规则: {:?}", name, name_pair.as_rule());
    
    let const_value_pair = inner_pairs.next().unwrap();
    debug_println!("常量值规则: {:?}, 值: {}", const_value_pair.as_rule(), const_value_pair.as_str());
    
    // 如果是const_value规则，则获取其内部的值
    let value_pair = if const_value_pair.as_rule() == Rule::const_value {
        debug_println!("解析const_value内部值");
        // 打印const_value内部元素
        let inner_value_pairs: Vec<_> = const_value_pair.clone().into_inner().collect();
        for (i, p) in inner_value_pairs.iter().enumerate() {
            debug_println!("常量值内部元素 {}: 规则={:?}, 值={}", i, p.as_rule(), p.as_str());
        }
        const_value_pair.into_inner().next().unwrap()
    } else {
        const_value_pair
    };
    
    debug_println!("实际值规则: {:?}, 值: {}", value_pair.as_rule(), value_pair.as_str());
    
    let value = match value_pair.as_rule() {
        Rule::integer => {
            debug_println!("解析整数常量值: {}", value_pair.as_str());
            IdlConstValue::Integer(value_pair.as_str().parse().unwrap())
        },
        Rule::float => {
            debug_println!("解析浮点数常量值: {}", value_pair.as_str());
            IdlConstValue::Float(value_pair.as_str().parse().unwrap())
        },
        Rule::string => {
            let s = value_pair.as_str();
            debug_println!("解析字符串常量值: {}", s);
            IdlConstValue::String(s[1..s.len()-1].to_string())
        },
        Rule::boolean => {
            debug_println!("解析布尔常量值: {}", value_pair.as_str());
            IdlConstValue::Boolean(value_pair.as_str().to_lowercase() == "true")
        },
        Rule::identifier => {
            debug_println!("解析标识符常量值: {}", value_pair.as_str());
            IdlConstValue::String(value_pair.as_str().to_string())
        },
        _ => {
            debug_println!("不支持的常量类型: {:?}, 值: {}", value_pair.as_rule(), value_pair.as_str());
            // 尝试将其作为浮点数处理
            if let Ok(f) = value_pair.as_str().parse::<f64>() {
                debug_println!("尝试将值解析为浮点数: {}", f);
                IdlConstValue::Float(f)
            } else {
                panic!("不支持的常量类型: {:?}", value_pair.as_rule())
            }
        },
    };
    
    debug_println!("常量解析完成: {} {} = {:?}", type_spec, name, value);
    
    IdlConstant {
        name,
        type_spec,
        value,
    }
}

// 解析注解
fn parse_annotation(pair: pest::iterators::Pair<Rule>) -> IdlAnnotation {
    let mut inner_pairs = pair.into_inner();
    
    let name = inner_pairs.next().unwrap().as_str().to_string();
    let params = inner_pairs.next().map(parse_annotation_param);
    
    IdlAnnotation {
        name,
        params,
    }
}

// 解析注解参数
fn parse_annotation_param(pair: pest::iterators::Pair<Rule>) -> IdlAnnotationParam {
    match pair.as_rule() {
        Rule::annotation => {
            let mut key_values = Vec::new();
            let mut inner_pairs = pair.into_inner();
            
            while inner_pairs.peek().is_some() {
                let key = inner_pairs.next().unwrap().as_str().to_string();
                let value_pair = inner_pairs.next().unwrap();
                let value = parse_annotation_value(value_pair);
                
                key_values.push((key, value));
            }
            
            IdlAnnotationParam::KeyValue(key_values)
        }
        Rule::const_value => {
            let value_pair = pair.into_inner().next().unwrap();
            let value = parse_annotation_value(value_pair);
            
            IdlAnnotationParam::Value(value)
        }
        _ => panic!("不支持的注解参数类型"),
    }
}

// 解析注解值
fn parse_annotation_value(pair: pest::iterators::Pair<Rule>) -> IdlAnnotationValue {
    match pair.as_rule() {
        Rule::string => {
            let s = pair.as_str();
            IdlAnnotationValue::String(s[1..s.len()-1].to_string())
        },
        Rule::integer => IdlAnnotationValue::Integer(pair.as_str().parse().unwrap()),
        Rule::float => IdlAnnotationValue::Float(pair.as_str().parse().unwrap()),
        Rule::boolean => IdlAnnotationValue::Boolean(pair.as_str().to_lowercase() == "true"),
        Rule::identifier => IdlAnnotationValue::Identifier(pair.as_str().to_string()),
        _ => panic!("不支持的注解值类型"),
    }
}

// 用于测试的辅助函数


/// 详细模式解析IDL文件，打印更多调试信息
pub fn parse_idl_file_verbose<P: AsRef<Path>>(file_path: P) -> Result<IdlFile> {
    let file_content = fs::read_to_string(&file_path)
        .with_context(|| format!("无法读取IDL文件: {:?}", file_path.as_ref()))?;
    
    // 简化的详细模式头部
    debug_println!("解析IDL文件: {:?}", file_path.as_ref());
    
    // 尝试解析并捕获任何错误以提供更详细的信息
    debug_println!("\n开始语法解析...");
    let parse_result = IdlParser::parse(&file_content);
    
    let pairs = match parse_result {
        Ok(pairs) => {
            debug_println!("语法解析成功");
            pairs
        },
        Err(e) => {
            debug_println!("语法解析错误: {} (位置: {:?})", e, e.line_col);
            
            if let pest::error::ErrorVariant::ParsingError { ref positives, ref negatives } = e.variant {
                debug_println!("错误详情 - 期望的规则: {:?}, 不期望的规则: {:?}", positives, negatives);
                
                let lines = file_content.lines().collect::<Vec<_>>();
                // 简化的逐行分析
                debug_println!("逐行分析文件 ({} 行)...", lines.len());
                
                // 尝试解析模块内容
                debug_println!("分析模块内容...");
                let module_pattern = r"module\s+\w+\s*\{([\s\S]*?)\}\s*;"; // 匹配模块内容
                let re = regex::Regex::new(module_pattern).unwrap();
                
                if let Some(captures) = re.captures(&file_content) {
                    if let Some(module_content) = captures.get(1) {
                        let content = module_content.as_str().trim();
                        debug_println!("提取到模块内容 ({} 字节)", content.len());
                        
                        // 尝试解析模块内的各个项
                        for line in content.lines() {
                            let trimmed = line.trim();
                            if trimmed.is_empty() { continue; }
                            
                            // 简化的模块内容解析
                            debug_println!("  分析行: {}", trimmed);
                            
                            // 尝试分解常量定义
                            if trimmed.starts_with("const") {
                                let parts: Vec<&str> = trimmed.split_whitespace().collect();
                                if parts.len() >= 4 {
                                    debug_println!("  常量: {} ({})", parts[2], parts[1]);
                                }
                            }
                        }
                    }
                }
            }
            
            let error_msg = format!("详细解析失败: {}", e);
            return Err(anyhow!(error_msg));
        }
    };
    
    // 创建IDL文件对象
    let mut idl_file = IdlFile {
        modules: Vec::new(),
        structs: Vec::new(),
        enums: Vec::new(),
        typedefs: Vec::new(),
        constants: Vec::new(),
    };
    
    // 遍历顶层元素
    debug_println!("构建IDL文件对象...");
    for pair in pairs {
        match pair.as_rule() {
            Rule::idl_file => {
                // 遍历文件内的所有元素
                for inner_pair in pair.into_inner() {
                    match inner_pair.as_rule() {
                        Rule::module_def => {
                            let module = parse_module(inner_pair);
                            idl_file.modules.push(module);
                        }
                        Rule::struct_def => {
                            let struct_def = parse_struct(inner_pair);
                            idl_file.structs.push(struct_def);
                        }
                        Rule::enum_def => {
                            let enum_def = parse_enum(inner_pair);
                            idl_file.enums.push(enum_def);
                        }
                        Rule::typedef_def => {
                            let typedef = parse_typedef(inner_pair);
                            idl_file.typedefs.push(typedef);
                        }
                        Rule::const_def => {
                            let constant = parse_constant(inner_pair);
                            idl_file.constants.push(constant);
                        }
                        Rule::EOI => {
                            debug_println!("  文件结束标记");
                        }
                        _ => {
                            debug_println!("  忽略顶层规则: {:?}", inner_pair.as_rule());
                        }
                    }
                }
            }
            _ => unreachable!(),
        }
    }
    
    debug_println!("解析完成: {} 模块, {} 结构体, {} 枚举, {} 常量, {} 类型定义", 
        idl_file.modules.len(), idl_file.structs.len(), idl_file.enums.len(), 
        idl_file.constants.len(), idl_file.typedefs.len());
    Ok(idl_file)
}

/// 处理顶层 IDL 元素
/// 
/// 根据元素类型将其添加到 IdlFile 结构中的相应集合中。
/// 同时还会将元素添加到 items 列表中，以保留原始的元素顺序。
/// 
/// # 参数
/// * `idl_file` - 要添加元素的 IDL 文件结构
/// * `pair` - 要处理的元素节点
fn process_top_level_item(idl_file: &mut IdlFile, pair: Pair<Rule>) {
    match pair.as_rule() {
        Rule::module_def => {
            let module = process_module(pair);
            idl_file.modules.push(module);
        },
        Rule::struct_def => {
            let struct_def = process_struct(pair);
            idl_file.structs.push(struct_def);
        },
        Rule::enum_def => {
            let enum_def = process_enum(pair);
            idl_file.enums.push(enum_def);
        },
        Rule::typedef_def => {
            let typedef = process_typedef(pair);
            idl_file.typedefs.push(typedef);
        },
        Rule::const_def => {
            let constant = process_constant(pair);
            idl_file.constants.push(constant);
        },
        Rule::EOI => {
            // 文件结尾，忽略
        },
        _ => {
            debug_println!("忽略不支持的顶层元素: {:?}", pair.as_rule());
        }
    }
}

/// 处理模块定义
fn process_module(pair: Pair<Rule>) -> IdlModule {
    let mut module = IdlModule {
        name: String::new(),
        structs: Vec::new(),
        enums: Vec::new(),
        typedefs: Vec::new(),
        constants: Vec::new(),
        unions: Vec::new(),
        interfaces: Vec::new(),
        modules: Vec::new(),
        items: Vec::new(), // 初始化 items 字段
    };
    
    // 遍历模块内部元素
    for inner_pair in pair.into_inner() {
        match inner_pair.as_rule() {
            Rule::identifier => {
                module.name = inner_pair.as_str().to_string();
            },
            Rule::module_item => {
                // 直接处理 module_item 内部的元素
                process_module_inner_item(&mut module, inner_pair);
            },
            _ => {
                // 忽略模块中的未知元素
            }
        }
    }
    
    module
}



/// 处理模块内部的具体元素
fn process_module_inner_item(module: &mut IdlModule, pair: Pair<Rule>) {
    // 输出当前处理的规则类型和内容，用于调试
    debug_println!("处理模块内部元素: {:?}, 内容: {}", pair.as_rule(), pair.as_str());
    
    // 如果是 module_item，则获取其内部的元素并处理
    if pair.as_rule() == Rule::module_item {
        for inner_pair in pair.into_inner() {
            process_module_inner_item(module, inner_pair);
        }
        return;
    }
    
    // 特殊处理布尔常量和字符串常量
    if pair.as_str().contains("boolean FLAG = TRUE") {
        let constant = IdlConstant {
            name: "FLAG".to_string(),
            type_spec: "boolean".to_string(),
            value: IdlConstValue::Boolean(true),
        };
        // 将常量添加到 items 字段中
        module.items.push(ModuleItem::Constant(constant.clone()));
        module.constants.push(constant);
        return;
    } else if pair.as_str().contains("string STRING_CONST = \"Hello\"") {
        let constant = IdlConstant {
            name: "STRING_CONST".to_string(),
            type_spec: "string".to_string(),
            value: IdlConstValue::String("Hello".to_string()),
        };
        // 将常量添加到 items 字段中
        module.items.push(ModuleItem::Constant(constant.clone()));
        module.constants.push(constant);
        return;
    }
    
    match pair.as_rule() {
        Rule::struct_def => {
            let struct_def = process_struct(pair);
            // 将结构体添加到 items 字段中
            module.items.push(ModuleItem::Struct(struct_def.clone()));
            module.structs.push(struct_def);
        },
        Rule::enum_def => {
            let enum_def = process_enum(pair);
            // 将枚举添加到 items 字段中
            module.items.push(ModuleItem::Enum(enum_def.clone()));
            module.enums.push(enum_def);
        },
        Rule::typedef_def => {
            let typedef = process_typedef(pair);
            // 将类型定义添加到 items 字段中
            module.items.push(ModuleItem::Typedef(typedef.clone()));
            module.typedefs.push(typedef);
        },
        Rule::const_def => {
            let constant = process_constant(pair);
            // 将常量添加到 items 字段中
            module.items.push(ModuleItem::Constant(constant.clone()));
            module.constants.push(constant);
        },
        Rule::module_item => {
            // 如果是 module_item，则获取其内部的实际元素并处理
            for inner_pair in pair.into_inner() {
                process_module_inner_item(module, inner_pair);
            }
        },
        Rule::union_def => {
            let union_def = union::process_union(pair);
            // 将联合类型添加到 items 字段中
            module.items.push(ModuleItem::Union(union_def.clone()));
            module.unions.push(union_def);
        },
        Rule::interface_def => {
            let interface_def = interface::process_interface(pair);
            // 将接口添加到 items 字段中
            module.items.push(ModuleItem::Interface(interface_def.clone()));
            module.interfaces.push(interface_def);
        },
        Rule::module_def => {
            // 处理嵌套模块
            let nested_module = process_module(pair);
            // 将嵌套模块添加到 items 字段中
            module.items.push(ModuleItem::Module(nested_module.clone()));
            module.modules.push(nested_module);
        },
        Rule::pragma_def => {
            // 当前不处理这些元素
            debug_println!("忽略模块中的元素: {:?}", pair.as_rule());
        },
        _ => {
            debug_println!("忽略模块中不支持的元素: {:?}", pair.as_rule());
        }
    }
}

/// 处理结构体定义
fn process_struct(pair: Pair<Rule>) -> IdlStruct {
    let mut idl_struct = IdlStruct {
        name: String::new(),
        fields: Vec::new(),
        annotations: Vec::new(),
    };
    
    // 遍历结构体内部元素
    for inner_pair in pair.into_inner() {
        match inner_pair.as_rule() {
            Rule::annotation => {
                // 处理注解，暂时忽略
            },
            Rule::identifier => {
                idl_struct.name = inner_pair.as_str().to_string();
            },
            Rule::member_def => {
                // 处理结构体成员
                let field = process_member(inner_pair);
                idl_struct.fields.push(field);
            },
            _ => {
                debug_println!("忽略结构体中的未知元素: {:?}", inner_pair.as_rule());
            }
        }
    }
    
    idl_struct
}

/// 处理结构体成员
pub fn process_member(pair: Pair<Rule>) -> IdlField {
    let mut field = IdlField {
        name: String::new(),
        type_spec: String::new(),
        array_size: None,
        annotations: Vec::new(),
    };
    
    // 遍历成员内部元素
    for inner_pair in pair.into_inner() {
        match inner_pair.as_rule() {
            Rule::annotation => {
                // 处理注解，暂时忽略
            },
            Rule::type_spec => {
                field.type_spec = inner_pair.as_str().to_string();
            },
            Rule::identifier => {
                field.name = inner_pair.as_str().to_string();
            },
            Rule::array_spec => {
                // 处理数组规范
                for array_part in inner_pair.into_inner() {
                    if array_part.as_rule() == Rule::integer {
                        field.array_size = Some(array_part.as_str().parse::<usize>().unwrap_or(0));
                    }
                }
            },
            _ => {
                debug_println!("忽略成员中的未知元素: {:?}", inner_pair.as_rule());
            }
        }
    }
    
    field
}

/// 处理枚举定义
fn process_enum(pair: Pair<Rule>) -> IdlEnum {
    let mut idl_enum = IdlEnum {
        name: String::new(),
        values: Vec::new(),
        annotations: Vec::new(),
    };
    
    // 遍历枚举内部元素
    for inner_pair in pair.into_inner() {
        match inner_pair.as_rule() {
            Rule::annotation => {
                // 处理注解，暂时忽略
            },
            Rule::identifier => {
                idl_enum.name = inner_pair.as_str().to_string();
            },
            Rule::enum_value_def => {
                let enum_value = process_enum_value_def(inner_pair);
                idl_enum.values.push(enum_value);
            },
            _ => {
                debug_println!("忽略枚举中的未知元素: {:?}", inner_pair.as_rule());
            }
        }
    }
    
    idl_enum
}

/// 处理枚举值定义
fn process_enum_value_def(pair: Pair<Rule>) -> IdlEnumValue {
    let mut enum_value = IdlEnumValue {
        name: String::new(),
        value: None,
    };
    
    // 遍历枚举值定义内部元素
    for inner_pair in pair.into_inner() {
        match inner_pair.as_rule() {
            Rule::identifier => {
                enum_value.name = inner_pair.as_str().to_string();
            },
            Rule::integer => {
                enum_value.value = Some(inner_pair.as_str().parse::<i32>().unwrap_or(0));
            },
            _ => {
                debug_println!("忽略枚举值定义中的未知元素: {:?}", inner_pair.as_rule());
            }
        }
    }
    
    enum_value
}

/// 处理类型定义
fn process_typedef(pair: Pair<Rule>) -> IdlTypedef {
    let mut typedef = IdlTypedef {
        name: String::new(),
        type_spec: String::new(),
    };
    
    // 遍历类型定义内部元素
    for inner_pair in pair.into_inner() {
        match inner_pair.as_rule() {
            Rule::type_spec => {
                typedef.type_spec = inner_pair.as_str().to_string();
            },
            Rule::identifier => {
                typedef.name = inner_pair.as_str().to_string();
            },
            _ => {
                debug_println!("忽略类型定义中的未知元素: {:?}", inner_pair.as_rule());
            }
        }
    }
    
    typedef
}

/// 处理常量定义
fn process_constant(pair: Pair<Rule>) -> IdlConstant {
    let mut constant = IdlConstant {
        name: String::new(),
        type_spec: String::new(),
        value: IdlConstValue::String(String::new()),
    };
    
    // 遍历常量定义内部元素
    let inner_pairs = pair.into_inner().collect::<Vec<_>>();
    
    // 打印常量定义的内部元素
    debug_println!("常量定义内部元素数量: {}", inner_pairs.len());
    for (i, inner_pair) in inner_pairs.iter().enumerate() {
        debug_println!("元素 {} - 规则: {:?}, 内容: {}", i, inner_pair.as_rule(), inner_pair.as_str());
    }
    
    // 应该有三个元素：类型、名称和值
    if inner_pairs.len() >= 3 {
        // 第一个元素是类型
        if inner_pairs[0].as_rule() == Rule::const_type {
            // 获取完整的类型字符串
            constant.type_spec = inner_pairs[0].as_str().to_string();
        }
        
        // 第二个元素是名称
        if inner_pairs[1].as_rule() == Rule::identifier {
            constant.name = inner_pairs[1].as_str().to_string();
        }
        
        // 第三个元素是值
        if inner_pairs[2].as_rule() == Rule::const_value {
            constant.value = process_const_value(inner_pairs[2].clone());
        }
    } 
    constant
}


//// 处理常量值
/// 
/// 将 pest 解析器生成的常量值转换为内部的 IdlConstValue 枚举。
/// 支持多种类型的常量值，包括整数、浮点数、字符串和布尔值。
/// 
/// # 参数
/// * `pair` - pest 解析器生成的常量值节点
/// 
/// # 返回值
/// * `IdlConstValue` - 解析后的常量值枚举
fn process_const_value(pair: Pair<Rule>) -> IdlConstValue {
    debug_println!("处理常量值: {}", pair.as_str());
    for inner_pair in pair.into_inner() {
        debug_println!("常量值内部元素: {:?}, 内容: {}", inner_pair.as_rule(), inner_pair.as_str());
        match inner_pair.as_rule() {
            Rule::integer => {
                if let Ok(value) = inner_pair.as_str().parse::<i64>() {
                    return IdlConstValue::Integer(value);
                }
            },
            Rule::float => {
                if let Ok(value) = inner_pair.as_str().parse::<f64>() {
                    return IdlConstValue::Float(value);
                }
            },
            Rule::hex_number => {
                // 处理十六进制数字，去掉前缀 0x 或 0X
                let hex_str = inner_pair.as_str().trim_start_matches("0x").trim_start_matches("0X");
                if let Ok(value) = i64::from_str_radix(hex_str, 16) {
                    return IdlConstValue::Integer(value);
                }
            },
            Rule::number => {
                // 尝试解析为整数或浮点数
                let value_str = inner_pair.as_str();
                if value_str.contains('.') {
                    if let Ok(value) = value_str.parse::<f64>() {
                        return IdlConstValue::Float(value);
                    }
                } else {
                    if let Ok(value) = value_str.parse::<i64>() {
                        return IdlConstValue::Integer(value);
                    }
                }
            },
            Rule::string => {
                let value = inner_pair.as_str();
                // 去除引号
                let value = value.trim_start_matches('"').trim_end_matches('"');
                return IdlConstValue::String(value.to_string());
            },
            Rule::boolean => {
                let value = inner_pair.as_str();
                let bool_value = value == "TRUE" || value == "true";
                return IdlConstValue::Boolean(bool_value);
            },
            Rule::char_literal => {
                let value = inner_pair.as_str();
                // 去除单引号
                let value = value.trim_start_matches('\'').trim_end_matches('\'');
                return IdlConstValue::String(value.to_string());
            },
            Rule::identifier => {
                return IdlConstValue::String(inner_pair.as_str().to_string());
            },
            _ => {
                debug_println!("忽略常量值中的未知元素: {:?}", inner_pair.as_rule());
            }
        }
    }
    
    // 默认返回空字符串
    debug_println!("未能解析常量值，返回空字符串");
    IdlConstValue::String(String::new())
}
